﻿/**
* 偽物3次元ベジェ曲線
* @author nutsu
*/

package {
	
	import flash.display.Sprite;
	import flash.display.Shape;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	public class Fake3DBezje extends Sprite{
		
		//始点
		private var handle_start:Sprite;
		//終点
		private var handle_end:Sprite;
		//コントロール点
		private var handle_ctrl0:Sprite;
		//コントロール点
		private var handle_ctrl1:Sprite;
		
		//偽コントロール点
		private var handle_fake0:Shape;
		private var handle_fake1:Shape;
		private var handle_fake2:Shape;
		private var f0:Point;
		private var f1:Point;
		private var f2:Point;
		
		//編集中か
		private var now_editting:Boolean = false;
		private var edit_handle:Sprite;
		
		/**
		* コンストラクタ
		*/
		public function Fake3DBezje(){
			super();
			
			//表示要素初期化
			handle_start = makeHandle();
			handle_end   = makeHandle();
			handle_ctrl0 = makeHandle();
			handle_ctrl1 = makeHandle();
			handle_fake0 = makePointShape( 0x999999, 2 );
			handle_fake1 = makePointShape( 0x999999, 2 );
			handle_fake2 = makePointShape( 0x999999, 2 );
			
			addChild( handle_start );
			addChild( handle_end );
			addChild( handle_ctrl0 );
			addChild( handle_ctrl1 );
			addChild( handle_fake0 );
			addChild( handle_fake1 );
			addChild( handle_fake2 );
			
			//初期位置
			handle_start.x = 75;
			handle_start.y = 250;
			handle_end.x = 350;
			handle_end.y = 250;
			handle_ctrl0.x = 100;
			handle_ctrl0.y = 120;
			handle_ctrl1.x = 325;
			handle_ctrl1.y = 120;
			
			draw();
			
			stage.addEventListener( MouseEvent.MOUSE_UP, stopEdit );
			stage.addEventListener( MouseEvent.MOUSE_MOVE, update );
			
		}
		
		/**
		*	描画
		*/
		private function draw():void{
			graphics.clear();
			
			//補助線
			graphics.lineStyle( 0, 0x666666 );
			graphics.moveTo( handle_start.x, handle_start.y );
			graphics.lineTo( handle_ctrl0.x, handle_ctrl0.y );
			graphics.moveTo( handle_end.x, handle_end.y );
			graphics.lineTo( handle_ctrl1.x, handle_ctrl1.y );
			
			//本物3次元ベジェ
			graphics.lineStyle( 0, 0xffffff );
			draw3DBezje( handle_start.x, handle_start.y, handle_end.x, handle_end.y, 
						 handle_ctrl0.x, handle_ctrl0.y, handle_ctrl1.x, handle_ctrl1.y, 50 );
			
			
			//偽物ベジェ曲線
			graphics.lineStyle( 0, 0x0099ff );
			drawFake3DBezje( handle_start.x, handle_start.y, handle_end.x, handle_end.y, 
						 handle_ctrl0.x, handle_ctrl0.y, handle_ctrl1.x, handle_ctrl1.y );
			
			//偽補助線
			graphics.lineStyle( 0, 0x333333 );
			graphics.moveTo( f0.x, f0.y );
			graphics.lineTo( f1.x, f1.y );
			handle_fake0.x = f0.x;
			handle_fake0.y = f0.y;
			handle_fake1.x = f1.x;
			handle_fake1.y = f1.y;
			handle_fake2.x = f2.x;
			handle_fake2.y = f2.y;
		}
		
		/**
		 * 3次元ベジェ曲線　偽
		 */
		private function drawFake3DBezje( x0:Number,y0:Number,x1:Number,y1:Number,cx0:Number,cy0:Number,cx1:Number,cy1:Number ):void{
			//始点とコントロール点0の0.25点
			f0 = Point.interpolate(new Point(x0,y0), new Point(cx0,cy0), 0.25);
			//終点とコントロール点1の0.25点
			f1 = Point.interpolate(new Point(x1,y1), new Point(cx1,cy1), 0.25);
			//さらに中点
			f2 = Point.interpolate( f0, f1, 0.5 );
			//描画
			graphics.moveTo( x0, y0 );
			graphics.curveTo( f0.x, f0.y, f2.x, f2.y );
			graphics.curveTo( f1.x, f1.y, x1, y1 );
		}
		
		/**
		 * 3次元ベジェ曲線
		 */
		private function draw3DBezje( x0:Number,y0:Number,x1:Number,y1:Number,cx0:Number,cy0:Number,cx1:Number,cy1:Number ,step:int=100):void{
			var k:Number = 1.0/step;
			graphics.moveTo(x0,y0);
			for( var i:int=1;i<=step;i++ ){
				var t:Number = i*k;
				var tp:Number = 1.0 - t;
				graphics.lineTo(	x0*tp*tp*tp + 3*cx0*t*tp*tp + 3*cx1*t*t*tp + x1*t*t*t ,
									y0*tp*tp*tp + 3*cy0*t*tp*tp + 3*cy1*t*t*tp + y1*t*t*t );
			}
		}
		
		/**
		* リスナ
		*/
		private function startEdit( e:MouseEvent ):void{
			now_editting = true;
			edit_handle = (e.target as Sprite);
			edit_handle.startDrag(false, new Rectangle(0,0,stage.stageWidth,stage.stageHeight));
		}
		
		private function stopEdit( e:MouseEvent ):void{
			now_editting = false;
			edit_handle.stopDrag();
			draw();
		}
		
		private function update( e:MouseEvent ):void{
			if( now_editting ) draw();
		}
		
		/**
		* ハンドル生成
		*/
		private function makeHandle():Sprite{
			var s:Sprite;
			(s = new Sprite()).addChild( makePointShape( 0xffffff, 4 ) );
			s.addEventListener( MouseEvent.MOUSE_DOWN, startEdit );
			s.buttonMode = true;
			return s;
		}
		
		/**
		* ポイント用Shape
		*/
		private function makePointShape(col:uint, size:Number):Shape{
			var d:Shape = new Shape();
			d.graphics.clear();
			d.graphics.beginFill( col );
			d.graphics.drawCircle( 0, 0, size );
			d.graphics.endFill();
			return d;
		}
	}
	
}

